// created by jay 0.7 (c) 1998 Axel.Schreiner@informatik.uni-osnabrueck.de

#line 2 "Parser.jay"
using System.Text;
using System.IO;
using System.Collections;
using System;

namespace Mono.Debugger.Frontends.CommandLine
{
	public class Parser
	{
		string name;
		ScriptingContext current_context;
		MyTextReader reader;
		bool do_quit = false;

		protected bool yacc_verbose_flag = false;

		public bool yacc_verbose {
			set {
				yacc_verbose_flag = value;
			}

			get {
				return yacc_verbose_flag;
			}
		}

		public bool Quit {
			get {
				return do_quit;
			}
		}

		Tokenizer lexer;

		public Tokenizer Lexer {
			get {
				return lexer;
			}
		}		   

#line 45 "-"

  /** simplified error message.
      @see <a href="#yyerror(java.lang.String, java.lang.String[])">yyerror</a>
    */
  public void yyerror (string message) {
    yyerror(message, null);
  }

  /** (syntax) error message.
      Can be overwritten to control message format.
      @param message text to be displayed.
      @param expected vector of acceptable tokens, if available.
    */
  public void yyerror (string message, string[] expected) {
    if ((expected != null) && (expected.Length  > 0)) {
      System.Console.Write (message+", expecting");
      for (int n = 0; n < expected.Length; ++ n)
        System.Console.Write (" "+expected[n]);
        System.Console.WriteLine ();
    } else
      System.Console.WriteLine (message);
  }

  /** debugging support, requires the package jay.yydebug.
      Set to null to suppress debugging messages.
    */
  protected yydebug.yyDebug debug;

  protected static  int yyFinal = 2;
  public static  string [] yyRule = {
    "$accept : command_loop",
    "command_loop :",
    "command_loop : command_loop command EOL",
    "command_loop : command_loop EOL",
    "command_loop : command_loop EOF",
    "command_loop : QUIT EOL",
    "command : CONTINUE opt_process",
    "command : STEP opt_process",
    "command : NEXT opt_process",
    "command : STEPI opt_process",
    "command : NEXTI opt_process",
    "command : FINISH opt_process",
    "command : BACKTRACE opt_process",
    "command : UP opt_process",
    "command : DOWN opt_process",
    "command : SHOW PROCESSES",
    "command : SHOW REGISTERS opt_process opt_frame_number",
    "command : SHOW PARAMETERS opt_process opt_frame_number",
    "command : SHOW LOCALS opt_process opt_frame_number",
    "command : SHOW MODULES",
    "command : SHOW TYPE variable_expression",
    "command : BREAK opt_process function_name",
    "command : BREAK opt_process STRING INTEGER",
    "command : PRINT expression",
    "command : FRAME opt_process opt_frame_number",
    "command : DISASSEMBLE opt_process opt_frame_number",
    "command : DISASSEMBLE METHOD opt_process opt_frame_number",
    "command : START INTEGER program_arguments",
    "command : START program_arguments",
    "command : PROCESS opt_process",
    "command : BACKGROUND opt_process",
    "command : STOP opt_process",
    "command : SET BANG IDENTIFIER ASSIGN variable_expression",
    "command : MODULE number_list module_operations",
    "command : MODULE ALL module_operations",
    "command : assert_command",
    "kind : UNKNOWN",
    "kind : FUNDAMENTAL",
    "kind : ARRAY",
    "kind : STRUCT",
    "kind : CLASS",
    "kind : POINTER",
    "assert_command : ASSERT KIND kind variable_expression",
    "assert_command : ASSERT ACCESSIBLE variable_expression",
    "assert_command : ASSERT NOT ACCESSIBLE variable_expression",
    "assert_command : ASSERT TYPE STRING variable_expression",
    "assert_command : ASSERT CONTENTS STRING variable_expression",
    "assert_command : ASSERT LINE INTEGER",
    "expression : opt_process opt_frame_number PERCENT IDENTIFIER",
    "expression : LENGTH OPEN_PARENS variable_expression CLOSE_PARENS",
    "expression : LOWER OPEN_PARENS variable_expression CLOSE_PARENS",
    "expression : UPPER OPEN_PARENS variable_expression CLOSE_PARENS",
    "expression : NUMBER",
    "expression : INTEGER",
    "expression : expression binary_operator expression",
    "expression : OPEN_PARENS expression CLOSE_PARENS",
    "expression : variable_expression",
    "variable_expression : STAR variable_expression",
    "variable_expression : BANG IDENTIFIER",
    "variable_expression : opt_process opt_frame_number DOLLAR IDENTIFIER",
    "variable_expression : variable_expression DOT IDENTIFIER",
    "variable_expression : variable_expression OPEN_BRACKET expression CLOSE_BRACKET",
    "variable_expression : PARENT OPEN_PARENS variable_expression CLOSE_PARENS",
    "program_arguments : program_arglist",
    "program_arglist : STRING",
    "program_arglist : program_arglist STRING",
    "opt_process :",
    "opt_process : AT INTEGER",
    "opt_frame_number :",
    "opt_frame_number : HASH INTEGER",
    "function_name : IDENTIFIER",
    "function_name : STRING",
    "number_list : INTEGER",
    "number_list : number_list COMMA INTEGER",
    "module_operation : IGNORE",
    "module_operation : UNIGNORE",
    "module_operation : BANG IGNORE",
    "module_operation : STEP",
    "module_operation : BANG STEP",
    "module_operations : module_operation",
    "module_operations : module_operations module_operation",
    "binary_operator : STAR",
    "binary_operator : PLUS",
    "binary_operator : MINUS",
    "binary_operator : DIV",
  };
  protected static  string [] yyName = {    
    "end-of-file",null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,"QUIT","EOL","EOF","NONE","ERROR",
    "IDENTIFIER","INTEGER","NUMBER","STRING","HASH","AT","PERCENT",
    "DOLLAR","DOT","BANG","COMMA","ASSIGN","STAR","PLUS","MINUS","DIV",
    "OPEN_PARENS","CLOSE_PARENS","OPEN_BRACKET","CLOSE_BRACKET",
    "PROCESSES","REGISTERS","PARAMETERS","LOCALS","TYPE","METHOD",
    "PARENT","LENGTH","LOWER","UPPER","KIND","ACCESSIBLE","CONTENTS",
    "LINE","NOT","UNKNOWN","FUNDAMENTAL","ARRAY","STRUCT","CLASS",
    "POINTER","MODULES","FRAME","START","SHOW","PROCESS","CONTINUE",
    "BACKTRACE","UP","DOWN","STEP","NEXT","STEPI","NEXTI","FINISH",
    "BREAK","PRINT","DISASSEMBLE","SET","ASSERT","SELECT","BACKGROUND",
    "STOP","MODULE","IGNORE","UNIGNORE","ALL",
  };

  /** index-checked interface to yyName[].
      @param token single character or %token value.
      @return token name or [illegal] or [unknown].
    */
  public static string yyname (int token) {
    if ((token < 0) || (token > yyName.Length)) return "[illegal]";
    string name;
    if ((name = yyName[token]) != null) return name;
    return "[unknown]";
  }

  /** computes list of expected tokens on error by tracing the tables.
      @param state for which to compute the list.
      @return list of token names.
    */
  protected string[] yyExpecting (int state) {
    int token, n, len = 0;
    bool[] ok = new bool[yyName.Length];

    if ((n = yySindex[state]) != 0)
      for (token = n < 0 ? -n : 0;
           (token < yyName.Length) && (n+token < yyTable.Length); ++ token)
        if (yyCheck[n+token] == token && !ok[token] && yyName[token] != null) {
          ++ len;
          ok[token] = true;
        }
    if ((n = yyRindex[state]) != 0)
      for (token = n < 0 ? -n : 0;
           (token < yyName.Length) && (n+token < yyTable.Length); ++ token)
        if (yyCheck[n+token] == token && !ok[token] && yyName[token] != null) {
          ++ len;
          ok[token] = true;
        }

    string [] result = new string[len];
    for (n = token = 0; n < len;  ++ token)
      if (ok[token]) result[n++] = yyName[token];
    return result;
  }

  /** the generated parser, with debugging messages.
      Maintains a state and a value stack, currently with fixed maximum size.
      @param yyLex scanner.
      @param yydebug debug message writer implementing yyDebug, or null.
      @return result of the last reduction, if any.
      @throws yyException on irrecoverable parse error.
    */
  public Object yyparse (yyParser.yyInput yyLex, Object yyd)
				 {
    this.debug = (yydebug.yyDebug)yyd;
    return yyparse(yyLex);
  }

  /** initial size and increment of the state/value stack [default 256].
      This is not final so that it can be overwritten outside of invocations
      of yyparse().
    */
  protected int yyMax;

  /** executed at the beginning of a reduce action.
      Used as $$ = yyDefault($1), prior to the user-specified action, if any.
      Can be overwritten to provide deep copy, etc.
      @param first value for $1, or null.
      @return first.
    */
  protected Object yyDefault (Object first) {
    return first;
  }

  /** the generated parser.
      Maintains a state and a value stack, currently with fixed maximum size.
      @param yyLex scanner.
      @return result of the last reduction, if any.
      @throws yyException on irrecoverable parse error.
    */
  public Object yyparse (yyParser.yyInput yyLex)
				{
    if (yyMax <= 0) yyMax = 256;			// initial size
    int yyState = 0;                                   // state stack ptr
    int [] yyStates = new int[yyMax];	                // state stack 
    Object yyVal = null;                               // value stack ptr
    Object [] yyVals = new Object[yyMax];	        // value stack
    int yyToken = -1;					// current input
    int yyErrorFlag = 0;				// #tks to shift

    int yyTop = 0;
    goto skip;
    yyLoop:
    yyTop++;
    skip:
    for (;; ++ yyTop) {
      if (yyTop >= yyStates.Length) {			// dynamically increase
        int[] i = new int[yyStates.Length+yyMax];
        yyStates.CopyTo (i, 0);
        yyStates = i;
        Object[] o = new Object[yyVals.Length+yyMax];
        yyVals.CopyTo (o, 0);
        yyVals = o;
      }
      yyStates[yyTop] = yyState;
      yyVals[yyTop] = yyVal;
      if (debug != null) debug.push(yyState, yyVal);

      yyDiscarded: for (;;) {	// discarding a token does not change stack
        int yyN;
        if ((yyN = yyDefRed[yyState]) == 0) {	// else [default] reduce (yyN)
          if (yyToken < 0) {
            yyToken = yyLex.advance() ? yyLex.token() : 0;
            if (debug != null)
              debug.lex(yyState, yyToken, yyname(yyToken), yyLex.value());
          }
          if ((yyN = yySindex[yyState]) != 0 && ((yyN += yyToken) >= 0)
              && (yyN < yyTable.Length) && (yyCheck[yyN] == yyToken)) {
            if (debug != null)
              debug.shift(yyState, yyTable[yyN], yyErrorFlag-1);
            yyState = yyTable[yyN];		// shift to yyN
            yyVal = yyLex.value();
            yyToken = -1;
            if (yyErrorFlag > 0) -- yyErrorFlag;
            goto yyLoop;
          }
          if ((yyN = yyRindex[yyState]) != 0 && (yyN += yyToken) >= 0
              && yyN < yyTable.Length && yyCheck[yyN] == yyToken)
            yyN = yyTable[yyN];			// reduce (yyN)
          else
            switch (yyErrorFlag) {
  
            case 0:
              yyerror("syntax error", yyExpecting(yyState));
              if (debug != null) debug.error("syntax error");
              goto case 1;
            case 1: case 2:
              yyErrorFlag = 3;
              do {
                if ((yyN = yySindex[yyStates[yyTop]]) != 0
                    && (yyN += Token.yyErrorCode) >= 0 && yyN < yyTable.Length
                    && yyCheck[yyN] == Token.yyErrorCode) {
                  if (debug != null)
                    debug.shift(yyStates[yyTop], yyTable[yyN], 3);
                  yyState = yyTable[yyN];
                  yyVal = yyLex.value();
                  goto yyLoop;
                }
                if (debug != null) debug.pop(yyStates[yyTop]);
              } while (-- yyTop >= 0);
              if (debug != null) debug.reject();
              throw new yyParser.yyException("irrecoverable syntax error");
  
            case 3:
              if (yyToken == 0) {
                if (debug != null) debug.reject();
                throw new yyParser.yyException("irrecoverable syntax error at end-of-file");
              }
              if (debug != null)
                debug.discard(yyState, yyToken, yyname(yyToken),
  							yyLex.value());
              yyToken = -1;
              goto yyDiscarded;		// leave stack alone
            }
        }
        int yyV = yyTop + 1-yyLen[yyN];
        if (debug != null)
          debug.reduce(yyState, yyStates[yyV-1], yyN, yyRule[yyN], yyLen[yyN]);
        yyVal = yyDefault(yyV > yyTop ? null : yyVals[yyV]);
        switch (yyN) {
case 2:
#line 131 "Parser.jay"
  {
		((Command) yyVals[-1+yyTop]).Execute (current_context);
	  }
  break;
case 5:
#line 137 "Parser.jay"
  {
		do_quit = true;
		reader.Close ();
	  }
  break;
case 6:
#line 145 "Parser.jay"
  {
		yyVal = new StepCommand ((ProcessExpression) yyVals[0+yyTop], WhichStepCommand.Continue);
	  }
  break;
case 7:
#line 149 "Parser.jay"
  {
		yyVal = new StepCommand ((ProcessExpression) yyVals[0+yyTop], WhichStepCommand.Step);
	  }
  break;
case 8:
#line 153 "Parser.jay"
  {
		yyVal = new StepCommand ((ProcessExpression) yyVals[0+yyTop], WhichStepCommand.Next);
	  }
  break;
case 9:
#line 157 "Parser.jay"
  {
		yyVal = new StepCommand ((ProcessExpression) yyVals[0+yyTop], WhichStepCommand.StepInstruction);
	  }
  break;
case 10:
#line 161 "Parser.jay"
  {
		yyVal = new StepCommand ((ProcessExpression) yyVals[0+yyTop], WhichStepCommand.NextInstruction);
	  }
  break;
case 11:
#line 165 "Parser.jay"
  {
		yyVal = new StepCommand ((ProcessExpression) yyVals[0+yyTop], WhichStepCommand.Finish);
	  }
  break;
case 12:
#line 169 "Parser.jay"
  {
		yyVal = new BacktraceCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 13:
#line 173 "Parser.jay"
  {
		yyVal = new UpCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 14:
#line 177 "Parser.jay"
  {
		yyVal = new DownCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 15:
#line 181 "Parser.jay"
  {
		yyVal = new ShowProcessesCommand ();
	  }
  break;
case 16:
#line 185 "Parser.jay"
  {
		yyVal = new ShowRegistersCommand ((ProcessExpression) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 17:
#line 189 "Parser.jay"
  {
		yyVal = new ShowParametersCommand ((ProcessExpression) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 18:
#line 193 "Parser.jay"
  {
		yyVal = new ShowLocalsCommand ((ProcessExpression) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 19:
#line 197 "Parser.jay"
  {
		yyVal = new ShowModulesCommand ();
	  }
  break;
case 20:
#line 201 "Parser.jay"
  {
		yyVal = new ShowVariableTypeCommand ((VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 21:
#line 205 "Parser.jay"
  {
		yyVal = new BreakCommand ((ProcessExpression) yyVals[-1+yyTop], (string) yyVals[0+yyTop]);
	  }
  break;
case 22:
#line 209 "Parser.jay"
  {
		yyVal = new BreakCommand ((ProcessExpression) yyVals[-2+yyTop], (string) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 23:
#line 213 "Parser.jay"
  {
		yyVal = new PrintCommand ((Expression) yyVals[0+yyTop]);
	  }
  break;
case 24:
#line 217 "Parser.jay"
  {
		yyVal = new FrameCommand ((ProcessExpression) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 25:
#line 221 "Parser.jay"
  {
		yyVal = new DisassembleCommand ((ProcessExpression) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 26:
#line 225 "Parser.jay"
  {
		yyVal = new DisassembleMethodCommand ((ProcessExpression) yyVals[-1+yyTop], (int) yyVals[0+yyTop]);
	  }
  break;
case 27:
#line 229 "Parser.jay"
  {
		yyVal = new StartCommand ((string []) yyVals[0+yyTop], (int) yyVals[-1+yyTop]);
	  }
  break;
case 28:
#line 233 "Parser.jay"
  {
		yyVal = new StartCommand ((string []) yyVals[0+yyTop]);
	  }
  break;
case 29:
#line 237 "Parser.jay"
  {
		yyVal = new SelectProcessCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 30:
#line 241 "Parser.jay"
  {
		yyVal = new BackgroundProcessCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 31:
#line 245 "Parser.jay"
  {
		yyVal = new StopProcessCommand ((ProcessExpression) yyVals[0+yyTop]);
	  }
  break;
case 32:
#line 249 "Parser.jay"
  {
		yyVal = new ScriptingVariableAssignCommand ((string) yyVals[-2+yyTop], (VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 33:
#line 253 "Parser.jay"
  {
		int[] modules = new int [((ArrayList) yyVals[-1+yyTop]).Count];
		((ArrayList) yyVals[-1+yyTop]).CopyTo (modules, 0);
		ModuleOperation[] operations = new ModuleOperation [((ArrayList) yyVals[0+yyTop]).Count];
		((ArrayList) yyVals[0+yyTop]).CopyTo (operations, 0);
		yyVal = new ModuleOperationCommand (modules, operations);
	  }
  break;
case 34:
#line 261 "Parser.jay"
  {
		ModuleOperation[] operations = new ModuleOperation [((ArrayList) yyVals[-1+yyTop]).Count];
		((ArrayList) yyVals[-1+yyTop]).CopyTo (operations, 0);
		yyVal = new ModuleOperationCommand (operations);
	  }
  break;
case 35:
#line 267 "Parser.jay"
  {
		yyVal = yyVals[0+yyTop];
	  }
  break;
case 36:
#line 274 "Parser.jay"
  {
		yyVal = TargetObjectKind.Unknown;
	  }
  break;
case 37:
#line 278 "Parser.jay"
  {
		yyVal = TargetObjectKind.Fundamental;
	  }
  break;
case 38:
#line 282 "Parser.jay"
  {
		yyVal = TargetObjectKind.Array;
	  }
  break;
case 39:
#line 286 "Parser.jay"
  {
		yyVal = TargetObjectKind.Struct;
	  }
  break;
case 40:
#line 290 "Parser.jay"
  {
		yyVal = TargetObjectKind.Class;
	  }
  break;
case 41:
#line 294 "Parser.jay"
  {
		yyVal = TargetObjectKind.Pointer;
	  }
  break;
case 42:
#line 301 "Parser.jay"
  {
		yyVal = new AssertKindCommand ((TargetObjectKind) yyVals[-1+yyTop], (VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 43:
#line 305 "Parser.jay"
  {
		yyVal = new AssertAccessibleCommand ((VariableExpression) yyVals[0+yyTop], true);
	  }
  break;
case 44:
#line 309 "Parser.jay"
  {
		yyVal = new AssertAccessibleCommand ((VariableExpression) yyVals[0+yyTop], false);
	  }
  break;
case 45:
#line 313 "Parser.jay"
  {
		yyVal = new AssertTypeCommand ((string) yyVals[-1+yyTop], (VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 46:
#line 317 "Parser.jay"
  {
		yyVal = new AssertContentsCommand ((string) yyVals[-1+yyTop], (VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 47:
#line 321 "Parser.jay"
  {
		yyVal = new AssertLineCommand ((int) yyVals[0+yyTop]);
	  }
  break;
case 48:
#line 328 "Parser.jay"
  {
		yyVal = new RegisterExpression ((ProcessExpression) yyVals[-3+yyTop], (int) yyVals[-2+yyTop], (string) yyVals[0+yyTop]);
	  }
  break;
case 49:
#line 332 "Parser.jay"
  {
		yyVal = new ArrayLengthExpression ((VariableExpression) yyVals[-1+yyTop]);
	  }
  break;
case 50:
#line 336 "Parser.jay"
  {
		yyVal = new ArrayLowerBoundExpression ((VariableExpression) yyVals[-1+yyTop]);
	  }
  break;
case 51:
#line 340 "Parser.jay"
  {
		yyVal = new ArrayUpperBoundExpression ((VariableExpression) yyVals[-1+yyTop]);
	  }
  break;
case 52:
#line 344 "Parser.jay"
  {
		yyVal = new NumberExpression ((int) yyVals[0+yyTop]);
	  }
  break;
case 53:
#line 348 "Parser.jay"
  {
		yyVal = new NumberExpression ((int) yyVals[0+yyTop]);
	  }
  break;
case 54:
#line 352 "Parser.jay"
  {
		yyVal = new BinaryOperator ((BinaryOperator.Kind) yyVals[-1+yyTop], (Expression) yyVals[-2+yyTop], (Expression) yyVals[0+yyTop]);
	  }
  break;
case 55:
#line 356 "Parser.jay"
  {
		yyVal = new ExpressionGroup ((Expression) yyVals[-1+yyTop]);
	  }
  break;
case 56:
#line 360 "Parser.jay"
  {
		yyVal = yyVals[0+yyTop];
	  }
  break;
case 57:
#line 367 "Parser.jay"
  {
		yyVal = new VariableDereferenceExpression ((VariableExpression) yyVals[0+yyTop]);
	  }
  break;
case 58:
#line 371 "Parser.jay"
  {
		yyVal = new ScriptingVariableReference ((string) yyVals[0+yyTop]);
	  }
  break;
case 59:
#line 375 "Parser.jay"
  {
		yyVal = new VariableReferenceExpression ((ProcessExpression) yyVals[-3+yyTop], (int) yyVals[-2+yyTop], (string) yyVals[0+yyTop]);
	  }
  break;
case 60:
#line 379 "Parser.jay"
  {
		yyVal = new StructAccessExpression ((VariableExpression) yyVals[-2+yyTop], (string) yyVals[0+yyTop]);
	  }
  break;
case 61:
#line 383 "Parser.jay"
  {
		yyVal = new ArrayAccessExpression ((VariableExpression) yyVals[-3+yyTop], (Expression) yyVals[-1+yyTop]);
	  }
  break;
case 62:
#line 387 "Parser.jay"
  {
		yyVal = new ParentClassExpression ((VariableExpression) yyVals[-1+yyTop]);
	  }
  break;
case 63:
#line 394 "Parser.jay"
  {
		ArrayList args = (ArrayList) yyVals[0+yyTop];
		string[] retval = new string [args.Count];
		args.CopyTo (retval, 0);
		yyVal = retval;
	  }
  break;
case 64:
#line 404 "Parser.jay"
  {
		ArrayList args = new ArrayList ();
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 65:
#line 411 "Parser.jay"
  {
		ArrayList args = (ArrayList) yyVals[-1+yyTop];
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 66:
#line 421 "Parser.jay"
  {
		yyVal = new ProcessExpression (-1);
	  }
  break;
case 67:
#line 425 "Parser.jay"
  {
		yyVal = new ProcessExpression ((int) yyVals[0+yyTop]);
	  }
  break;
case 68:
#line 432 "Parser.jay"
  {
		yyVal = -1;
	  }
  break;
case 69:
#line 436 "Parser.jay"
  {
		yyVal = yyVals[0+yyTop];
	  }
  break;
case 70:
#line 443 "Parser.jay"
  {
		yyVal = (string) yyVals[0+yyTop];
	  }
  break;
case 71:
#line 447 "Parser.jay"
  {
		yyVal = (string) yyVals[0+yyTop];
	  }
  break;
case 72:
#line 454 "Parser.jay"
  {
		ArrayList args = new ArrayList ();
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 73:
#line 461 "Parser.jay"
  {
		ArrayList args = (ArrayList) yyVals[-2+yyTop];
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 74:
#line 471 "Parser.jay"
  {
		yyVal = ModuleOperation.Ignore;
	  }
  break;
case 75:
#line 475 "Parser.jay"
  {
		yyVal = ModuleOperation.UnIgnore;
	  }
  break;
case 76:
#line 479 "Parser.jay"
  {
		yyVal = ModuleOperation.UnIgnore;
	  }
  break;
case 77:
#line 483 "Parser.jay"
  {
		yyVal = ModuleOperation.Step;
	  }
  break;
case 78:
#line 487 "Parser.jay"
  {
		yyVal = ModuleOperation.DontStep;
	  }
  break;
case 79:
#line 494 "Parser.jay"
  {
		ArrayList args = new ArrayList ();
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 80:
#line 501 "Parser.jay"
  {
		ArrayList args = (ArrayList) yyVals[-1+yyTop];
		args.Add (yyVals[0+yyTop]);

		yyVal = args;
	  }
  break;
case 81:
#line 511 "Parser.jay"
  {
		yyVal = BinaryOperator.Kind.Mult;
	  }
  break;
case 82:
#line 515 "Parser.jay"
  {
		yyVal = BinaryOperator.Kind.Plus;
	  }
  break;
case 83:
#line 519 "Parser.jay"
  {
		yyVal = BinaryOperator.Kind.Minus;
	  }
  break;
case 84:
#line 523 "Parser.jay"
  {
		yyVal = BinaryOperator.Kind.Div;
	  }
  break;
#line 873 "-"
        }
        yyTop -= yyLen[yyN];
        yyState = yyStates[yyTop];
        int yyM = yyLhs[yyN];
        if (yyState == 0 && yyM == 0) {
          if (debug != null) debug.shift(0, yyFinal);
          yyState = yyFinal;
          if (yyToken < 0) {
            yyToken = yyLex.advance() ? yyLex.token() : 0;
            if (debug != null)
               debug.lex(yyState, yyToken,yyname(yyToken), yyLex.value());
          }
          if (yyToken == 0) {
            if (debug != null) debug.accept(yyVal);
            return yyVal;
          }
          goto yyLoop;
        }
        if (((yyN = yyGindex[yyM]) != 0) && ((yyN += yyState) >= 0)
            && (yyN < yyTable.Length) && (yyCheck[yyN] == yyState))
          yyState = yyTable[yyN];
        else
          yyState = yyDgoto[yyM];
        if (debug != null) debug.shift(yyStates[yyTop], yyState);
	 goto yyLoop;
      }
    }
  }

   static  short [] yyLhs  = {              -1,
    0,    0,    0,    0,    0,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,   11,   11,   11,   11,   11,
   11,   10,   10,   10,   10,   10,   10,    6,    6,    6,
    6,    6,    6,    6,    6,    6,    4,    4,    4,    4,
    4,    4,    7,   13,   13,    2,    2,    3,    3,    5,
    5,    8,    8,   14,   14,   14,   14,   14,    9,    9,
   12,   12,   12,   12,
  };
   static  short [] yyLen = {           2,
    0,    3,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    4,    4,    4,    2,    3,
    3,    4,    2,    3,    3,    4,    3,    2,    2,    2,
    2,    5,    3,    3,    1,    1,    1,    1,    1,    1,
    1,    4,    3,    4,    4,    4,    3,    4,    4,    4,
    4,    1,    1,    3,    3,    1,    2,    2,    4,    3,
    4,    4,    1,    1,    2,    0,    2,    0,    2,    1,
    1,    1,    3,    1,    1,    2,    1,    2,    1,    2,
    1,    1,    1,    1,
  };
   static  short [] yyDefRed = {            0,
    0,    0,    5,    3,    4,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   35,    0,    0,
    0,   64,   28,    0,   15,    0,    0,    0,    0,   19,
   29,    6,   12,   13,   14,    7,    8,    9,   10,   11,
    0,   53,   52,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   30,   31,   72,    0,    0,    2,   67,    0,
   24,   27,   65,    0,    0,    0,    0,    0,   70,    0,
   21,   58,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   81,   82,   83,   84,    0,    0,   25,    0,    0,
   36,   37,   38,   39,   40,   41,    0,    0,    0,   47,
    0,    0,   77,   74,   75,    0,   79,    0,    0,   69,
   16,   17,   18,    0,   22,   55,    0,    0,    0,    0,
    0,    0,   60,    0,    0,   26,    0,    0,    0,    0,
    0,   78,   76,   80,   73,   62,   49,   50,   51,   48,
   59,   61,    0,
  };
  protected static  short [] yyDgoto  = {             2,
   27,   87,   81,   62,   91,   63,   33,   77,  126,   28,
  117,  106,   34,  127,
  };
  protected static  short [] yySindex = {         -242,
 -224, -221,    0,    0,    0, -161, -186, -166, -161, -161,
 -161, -161, -161, -161, -161, -161, -161, -161, -161, -135,
 -256, -190, -115, -161, -161, -261, -151,    0, -153, -155,
 -152,    0,    0, -150,    0, -161, -161, -161, -136,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -257,    0,    0, -139, -136, -135, -148, -145, -144, -138,
 -155, -210,  -70, -161, -155, -116, -114, -105, -136, -104,
 -113, -130,    0,    0,    0, -258, -265,    0,    0,  -94,
    0,    0,    0, -155, -155, -155, -155, -210,    0,  -89,
    0,    0, -210,  -76, -136, -136, -136, -136, -259,  -87,
 -135,    0,    0,    0,    0, -135, -155,    0, -125, -136,
    0,    0,    0,    0,    0,    0, -136, -210, -136,    0,
 -136, -277,    0,    0,    0, -258,    0,  -75, -258,    0,
    0,    0,    0,  -93,    0,    0, -229, -204, -158,  -97,
  -73,  -72,    0,  -90,  -70,    0, -136, -210, -210, -210,
 -210,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -210,
  };
  protected static  short [] yyRindex = {            1,
    0,    0,    0,    0,    0, -254,    0,    0,  -56,  -56,
  -56,  -56,  -56,  -56,  -56,  -56,  -56,  -56, -226, -124,
 -254,    0,    0,  -56,  -56,    0,    0,    0,    0,  -50,
    0,    0,    0,  -49,    0, -254, -254, -254, -214,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -214, -124,    0,    0,    0,    0,
 -195, -117,  -48, -254,  -50,    0,    0,    0, -214,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  -50,  -50,  -50,  -58,  -46,    0,  -45,
    0,    0, -109,    0, -214, -214, -214, -214,    0,    0,
 -124,    0,    0,    0,    0, -124,  -50,    0,    0, -214,
    0,    0,    0,    0,    0,    0, -214,  -44, -214,    0,
 -214,    0,    0,    0,    0,  -43,    0,    0,  -42,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -216,    0, -214,  -41,  -40,  -38,
  -37,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  -36,
  };
  protected static  short [] yyGindex = {            0,
    0,    8,   40,  -39,    0,  -53,  188,    0,  146,    0,
    0,    0,    0,  -86,
  };
  protected static  short [] yyTable = {            88,
    1,   75,   94,   66,   89,  122,  128,   90,  141,  142,
   29,   66,  122,   30,    1,   93,   41,   42,   43,   44,
   45,   46,   47,   48,   49,   50,   51,   61,   65,  118,
   64,   73,   74,    3,  152,   66,    4,    5,   66,  154,
  100,   54,  154,   84,   85,   86,  123,  144,  153,  156,
  101,   66,  145,  123,   66,  137,  138,  139,  140,  100,
  124,  125,   54,   61,   54,  100,   76,  124,  125,  101,
  148,  107,   68,   68,  157,  101,   31,  149,   32,  150,
   66,  151,    6,    7,    8,    9,   10,   11,   12,   13,
   14,   15,   16,   17,   18,   19,   20,   21,   22,   23,
   99,   24,   25,   26,  108,   29,   78,  163,   61,   79,
   80,  100,   32,   61,   83,   35,   36,   37,   38,   39,
  158,  101,   92,  131,  132,  133,  134,   52,   53,   95,
   29,   29,   96,   97,   54,   54,   40,   55,   55,   98,
   56,   66,   56,   66,   66,  109,  146,  147,   57,  120,
  110,   57,   57,   58,   59,   60,   56,   56,   56,   56,
  119,   56,  121,   56,   57,   57,   57,   57,  130,   57,
   67,   57,  100,  135,  143,  142,   68,   69,   70,   71,
   72,  159,  101,  102,  103,  104,  105,  155,  160,  161,
  162,  111,  112,  113,  114,  115,  116,  102,  103,  104,
  105,   66,  136,  102,  103,  104,  105,   68,   63,   23,
   68,   20,   71,   43,   34,   33,   45,   42,   82,   46,
   44,   32,  129,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    1,    1,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    0,    1,    1,    1,
  };
  protected static  short [] yyCheck = {            39,
    0,  263,   56,  258,  262,  271,  272,  265,  268,  269,
  267,  266,  271,    6,  257,   55,    9,   10,   11,   12,
   13,   14,   15,   16,   17,   18,   19,   20,   21,   69,
  287,   24,   25,  258,  312,  262,  258,  259,  265,  126,
  270,  258,  129,   36,   37,   38,  312,  101,  326,  279,
  280,  266,  106,  312,  269,   95,   96,   97,   98,  270,
  326,  327,  279,   56,  281,  270,  328,  326,  327,  280,
  110,   64,  268,  269,  279,  280,  263,  117,  265,  119,
  271,  121,  304,  305,  306,  307,  308,  309,  310,  311,
  312,  313,  314,  315,  316,  317,  318,  319,  320,  321,
   61,  323,  324,  325,   65,  267,  258,  147,  101,  263,
  266,  270,  265,  106,  265,  282,  283,  284,  285,  286,
  279,  280,  262,   84,   85,   86,   87,  263,  264,  278,
  267,  267,  278,  278,  271,  271,  303,  274,  274,  278,
  258,  266,  278,  268,  269,  262,  107,  273,  258,  263,
  265,  288,  288,  289,  290,  291,  274,  275,  276,  277,
  265,  279,  293,  281,  274,  275,  276,  277,  263,  279,
  286,  281,  270,  263,  262,  269,  292,  293,  294,  295,
  296,  279,  280,  274,  275,  276,  277,  263,  262,  262,
  281,  297,  298,  299,  300,  301,  302,  274,  275,  276,
  277,  258,  279,  274,  275,  276,  277,  258,  258,  258,
  269,  258,  258,  258,  258,  258,  258,  258,   31,  258,
  258,  258,   77,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,  259,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  304,  305,  306,  307,  308,  309,
  310,  311,  312,  313,  314,  315,  316,  317,  318,  319,
  320,  321,   -1,  323,  324,  325,
  };

#line 528 "Parser.jay"

public Parser (ScriptingContext context, string name)
{
	this.name = name;
	this.reader = new MyTextReader ();
	this.current_context = context;

	lexer = new Tokenizer (context, reader, name);
}

public bool Parse (string text)
{
	try {
		reader.Text = text;
		lexer.restart ();
		if (yacc_verbose_flag)
			yyparse (lexer, new yydebug.yyDebugSimple ());
		else
			yyparse (lexer);
		return true;
	} catch (Exception e){
		// Please do not remove this, it is used during debugging
		// of the grammar
		//
		current_context.Error (lexer.location + "  : Parsing error ");
		current_context.Error (e.ToString ());
		return false;
	}
}

/* end end end */
}

#line 1095 "-"
namespace yydebug {
        using System;
	 public interface yyDebug {
		 void push (int state, Object value);
		 void lex (int state, int token, string name, Object value);
		 void shift (int from, int to, int errorFlag);
		 void pop (int state);
		 void discard (int state, int token, string name, Object value);
		 void reduce (int from, int to, int rule, string text, int len);
		 void shift (int from, int to);
		 void accept (Object value);
		 void error (string message);
		 void reject ();
	 }
	 
	 class yyDebugSimple : yyDebug {
		 void println (string s){
			 Console.WriteLine (s);
		 }
		 
		 public void push (int state, Object value) {
			 println ("push\tstate "+state+"\tvalue "+value);
		 }
		 
		 public void lex (int state, int token, string name, Object value) {
			 println("lex\tstate "+state+"\treading "+name+"\tvalue "+value);
		 }
		 
		 public void shift (int from, int to, int errorFlag) {
			 switch (errorFlag) {
			 default:				// normally
				 println("shift\tfrom state "+from+" to "+to);
				 break;
			 case 0: case 1: case 2:		// in error recovery
				 println("shift\tfrom state "+from+" to "+to
					     +"\t"+errorFlag+" left to recover");
				 break;
			 case 3:				// normally
				 println("shift\tfrom state "+from+" to "+to+"\ton error");
				 break;
			 }
		 }
		 
		 public void pop (int state) {
			 println("pop\tstate "+state+"\ton error");
		 }
		 
		 public void discard (int state, int token, string name, Object value) {
			 println("discard\tstate "+state+"\ttoken "+name+"\tvalue "+value);
		 }
		 
		 public void reduce (int from, int to, int rule, string text, int len) {
			 println("reduce\tstate "+from+"\tuncover "+to
				     +"\trule ("+rule+") "+text);
		 }
		 
		 public void shift (int from, int to) {
			 println("goto\tfrom state "+from+" to "+to);
		 }
		 
		 public void accept (Object value) {
			 println("accept\tvalue "+value);
		 }
		 
		 public void error (string message) {
			 println("error\t"+message);
		 }
		 
		 public void reject () {
			 println("reject");
		 }
		 
	 }
}
// %token constants
 class Token {
  public const int QUIT = 257;
  public const int EOL = 258;
  public const int EOF = 259;
  public const int NONE = 260;
  public const int ERROR = 261;
  public const int IDENTIFIER = 262;
  public const int INTEGER = 263;
  public const int NUMBER = 264;
  public const int STRING = 265;
  public const int HASH = 266;
  public const int AT = 267;
  public const int PERCENT = 268;
  public const int DOLLAR = 269;
  public const int DOT = 270;
  public const int BANG = 271;
  public const int COMMA = 272;
  public const int ASSIGN = 273;
  public const int STAR = 274;
  public const int PLUS = 275;
  public const int MINUS = 276;
  public const int DIV = 277;
  public const int OPEN_PARENS = 278;
  public const int CLOSE_PARENS = 279;
  public const int OPEN_BRACKET = 280;
  public const int CLOSE_BRACKET = 281;
  public const int PROCESSES = 282;
  public const int REGISTERS = 283;
  public const int PARAMETERS = 284;
  public const int LOCALS = 285;
  public const int TYPE = 286;
  public const int METHOD = 287;
  public const int PARENT = 288;
  public const int LENGTH = 289;
  public const int LOWER = 290;
  public const int UPPER = 291;
  public const int KIND = 292;
  public const int ACCESSIBLE = 293;
  public const int CONTENTS = 294;
  public const int LINE = 295;
  public const int NOT = 296;
  public const int UNKNOWN = 297;
  public const int FUNDAMENTAL = 298;
  public const int ARRAY = 299;
  public const int STRUCT = 300;
  public const int CLASS = 301;
  public const int POINTER = 302;
  public const int MODULES = 303;
  public const int FRAME = 304;
  public const int START = 305;
  public const int SHOW = 306;
  public const int PROCESS = 307;
  public const int CONTINUE = 308;
  public const int BACKTRACE = 309;
  public const int UP = 310;
  public const int DOWN = 311;
  public const int STEP = 312;
  public const int NEXT = 313;
  public const int STEPI = 314;
  public const int NEXTI = 315;
  public const int FINISH = 316;
  public const int BREAK = 317;
  public const int PRINT = 318;
  public const int DISASSEMBLE = 319;
  public const int SET = 320;
  public const int ASSERT = 321;
  public const int SELECT = 322;
  public const int BACKGROUND = 323;
  public const int STOP = 324;
  public const int MODULE = 325;
  public const int IGNORE = 326;
  public const int UNIGNORE = 327;
  public const int ALL = 328;
  public const int yyErrorCode = 256;
 }
 namespace yyParser {
  using System;
  /** thrown for irrecoverable syntax errors and stack overflow.
    */
  public class yyException : System.Exception {
    public yyException (string message) : base (message) {
    }
  }

  /** must be implemented by a scanner object to supply input to the parser.
    */
  public interface yyInput {
    /** move on to next token.
        @return false if positioned beyond tokens.
        @throws IOException on input error.
      */
    bool advance (); // throws java.io.IOException;
    /** classifies current token.
        Should not be called if advance() returned false.
        @return current %token or single character.
      */
    int token ();
    /** associated with current token.
        Should not be called if advance() returned false.
        @return value for token().
      */
    Object value ();
  }
 }
} // close outermost namespace, that MUST HAVE BEEN opened in the prolog
